import time
import numpy as np

"""
Test system
"""

def eq(x,y):
    return np.allclose(x, y)

def assert_eq_results(result, answer):
    # Assuming np.arrays no deeper than second layer -> FIXME
    if isinstance(answer, tuple):
        assert(isinstance(result, tuple)), "Tuple awaited"
        assert(len(result) == len(answer)), \
            "Output tuples has different length"
        for (r, a) in zip(result, answer):
            assert_eq_results(r, a)
    else:
        assert(type(result) == type(answer)), "Outputs have different type"
        assert(eq(result, answer)), "Outputs differ"
    
def test(func, answer, args=None, kwargs=None, name=None):
    """
    Test a function given args, kwargs and correct answer.
    Argument 'name' specifies the test name (by default: function name).
    """
    if args == None:
        args = []
    if kwargs == None:
        kwargs = {}
    
    if name == None:
        name = func.__name__
    try:
        result = func(*args, **kwargs)
        assert_eq_results(result, answer)
    except AssertionError as e:
        print "***Test '%s' failed***" % name
        print "Function:", func.__name__
        if args != []: 
            print "Args:", args
        if kwargs != {}:
            print "Kwargs:", kwargs
        print "Reason:", e.message
        print "Expected:", answer
        print "Got:", result
    

def test_many(func, answer_list, args_list=None, kwargs_list=None):
    if args_list == None:
        args_list = [None] * len(answer_list)
    if kwargs_list == None:
        kwargs_list = [None] * len(answer_list)
    assert(len(answer_list) == len(args_list) == len(kwargs_list))
    
    for i in range(len(answer_list)):
        test(func, answer_list[i], args_list[i], kwargs_list[i])


class MyTimer:
    def __init__(self, start, quiet=False):
        self.start = start
        self.quiet = quiet
    def toc(self, quiet=False):
        t = time.time() - self.start
        if not self.quiet and not quiet:
            print "Elapsed %0.3f secs" % t
        return t
        
def tic(quiet=False):
    return MyTimer(time.time(), quiet=quiet)
